
import React, { useRef } from "react";
import { useDrag, useDrop } from "react-dnd";
import { cn } from "@/lib/utils";

// Define item types for drag and drop
export const ItemTypes = {
  CARD: "card",
  FEATURE: "feature",
  PREDICTION: "prediction",
};

export interface DraggableProps {
  id: string | number;
  index: number;
  type?: string;
  moveItem: (dragIndex: number, hoverIndex: number) => void;
  children: React.ReactNode;
  className?: string;
}

export const Draggable = ({
  id,
  index,
  type = ItemTypes.CARD,
  moveItem,
  children,
  className,
}: DraggableProps) => {
  const ref = useRef<HTMLDivElement>(null);

  // Set up the drag part
  const [{ isDragging }, drag] = useDrag({
    type,
    item: { id, index },
    collect: (monitor) => ({
      isDragging: monitor.isDragging(),
    }),
  });

  // Set up the drop part
  const [, drop] = useDrop({
    accept: type,
    hover(item: { id: string | number; index: number }, monitor) {
      if (!ref.current) {
        return;
      }
      
      const dragIndex = item.index;
      const hoverIndex = index;
      
      // Don't replace items with themselves
      if (dragIndex === hoverIndex) {
        return;
      }
      
      // Time to actually perform the action
      moveItem(dragIndex, hoverIndex);
      
      // Note: we're mutating the monitor item here!
      // Generally it's better to avoid mutations,
      // but it's good here for the sake of performance
      // to avoid expensive index searches.
      item.index = hoverIndex;
    },
  });

  // Connect the drag and drop refs
  drag(drop(ref));

  return (
    <div 
      ref={ref} 
      className={cn(
        "transition-opacity", 
        isDragging ? "opacity-50" : "opacity-100", 
        className
      )}
    >
      {children}
    </div>
  );
};

